/* SRE-DATA: The SRE-http data indexer: daemon launcher.

Invoking SRE-DATA:

  SREDATA can be called in one of 3 ways
 1) With  the CUSTOM_INITS parameter in   SREFILTR.80.  
   Actually, you can launch seperate versions of  SREDATA, for seperate
   "SRE-Data daemons"

  Examples:
    CUSTOM_INITS='  SREDATA index1 prog=index\index1.in ; preload '
    CUSTOM_INITS='  SREDATA index2 data=index\index2.dat '

 2) From the command line. I.e.;
        D:\GOSERVE>SREDATA name=index1 prog=index\index1.in monitor=1

 3) From a procedure (i.e.; an addon). I.e.;
        status=sredata('INDEX1','PROG','INDEX\INDEX1.IN')

When called, SREDATA expects a few arguments.

 1)  type=FILENAME.EXT ; where type is one of PROG,DATA,STEM,KEY, or KEYLOAD
 2)  NAME=ANAME : a "handle" you'll use when querying.

  * Type values are:
        PROG   = "program" for reading a data file. PROG files will contain
                  various options, and must contain a DATA statement (to
                identify the data file)
        DATA   = a data file (the file is read using defaults that a PROG
                 can override)
        STEM   = a "stem file"
        KEY    = a "key" file
        KEYLOAD  = a "key" file, to be loaded into memory
    the last three (STEM, KEY, KEYLOAD) are produced by the SREDATAX program.

  * If NAME= is not specified, then a value of INDEX  is assumed.

  * relative file names are assumed to be relative to the goserve working
    directory (i.e.; d:\goserve)

 3) Optional: MONITOR=1.  
          This should ONLEY BE be used when launched from the command line.
   When launched from a command line (of a seperate os/2 session), SREDATA.CMD
   will not die after launching the daemon -- it will stay alive
   and monitor it.  The main reason for doing this is to keep
   something happening in the session (thereby reminding you not to
   "throw it away").


When called,  SREDATA will do several things
    1) If SREF_FIND_INDEX does not exist in macrospace, it will
       load it (using the SREDATA.SRF file)
    2) It will then create a queue and a semaphore using the
       NAME (or using INDEX, if no NAME was specified).
    3) It will launch SREDATA.RXX as a thread, and give it
       the arguments.
    4) It will quit -- but SREDATA.RXX will continue (as a daemon)
        (or ,if MONITOR=1, it will go into a monitor mode).

THEREFORE: The real work is done by SREDATA.RXX -- the job
           of SREDATA.CMD (this file) is to launch the SREDATA.RXX as a
           thread, and to make sure that SREF_FIND_INDEX  (in SREDAT.SRF)
           is in macrospace.
*/
sredata:
parse arg stuff,v2,v3

call init_stuff

crlf='0d0a'x

aname='INDEX'
adata='' ; aprog=''
progstring=0
domonitor=0

parse source mookie
parse var mookie . ltype .
/* called from procedure? */
if ltype<>'COMMAND' then do
    aname=stuff
    file_type=strip(upper(v2))
    select
       when abbrev(file_type,'STR') then do
                aprog=v3 ; progstring=1 ; file_type='R'
       end  /* Do */
       when abbrev(file_type,'PROG')=1 then do
             aprog=v3 ; file_type='R'
       end
       when abbrev(file_type,'DATA')=1  then do
            adata=v3 ; file_type='R'
       end
       when abbrev(file_type,'STEM')+abbrev(file_type,'KEY') , 
                  +abbrev(file_type,'LOAD')>0 then  do
          file_type=left(file_type,1)
          adata=v3
       end
       otherwise  nop
    end

end  /* Do */

else do         /* command line */
  do until stuff=""
    parse var stuff v1  stuff
    if pos('=',v1)=' ' then v1='NAME='||v1
    parse var v1 atype '=' aval ; atype=upper(strip(atype)) ;aval=upper(strip(aval))
    select 
       when abbrev(atype,"NAME")=1  then do 
          aname=aval
       end  /* Do */
       when abbrev(atype,'PROG')=1 then do
          aprog=aval
          file_type='R'
       end
       when abbrev(atype,'DATA')=1 then do
          adata=aval
          file_type='R'
       end
       when abbrev(atype,'STEM')=1  then do
          adata=aval
          file_type='S'
       end
       when abbrev(atype,'KEY')=1  then do
          adata=aval
          file_type='K'
       end
       when abbrev(atype,'LOAD')=1  then do
          adata=aval
          file_type='L'
       end
       when atype="MONITOR" then do
           domonitor=aval
       end  /* Do */
    otherwise nop
    end  /* select */
  end /* do */
end

/* data or prog must be specified  -- prog takes priority. */
if adata="" & aprog="" then do
         call pmprintf(' SREDATA: error, must specify data file, or PROG file')
         say ' SREDATA: error, must specify data file, or PROG file'
 
         exit '0 no data of prog file '
end  /* Do */

foo=get_opts(aprog,progstring)             /* get PROG options, and get "regular" data file*/

if adata=' ' then do
    call pmprintf(' SREDATA: No datafile not specified ')
    say  ' SREDATA: No datafile not specified '
    exit ' 0 no datafile '
end  /* Do */
yow=stream(adata,'c','query exists')         /* check on data */
if yow="" then do
           call pmprintf(' SREDATA: datafile 'adata ' not found. ')
           say ' SREDATA: datafile 'adata ' not found. '
           exit '0 datafile not found '
end  /* Do */

/* see if ANAME semaphore & queue exists. If yes, then some other
   dataset is alive, so  ERROR   (ANAME must be unique!) */
 usequeue='SRED_'||aname
 usesem='\sem32\SRED_'||aname
 okay=make_semq(usesem,usequeue)
 if okay<>1 then do
    call pmprintf(' SREDATA: NAME= ' aname ' currently in use: ')
    say ' SREDATA: NAME= ' aname ' currently in use: '
    exit '0 Name: 'aname' in use '
 end  /* Do */

/* launch thread */

  if stream('SREDATA.RXX','c','query exists')=' ' then do
       call pmprintf(' SREDATA: SREDATA.RXX not found ')
       say ' SREDATA: SREDATA.RXX not found '
       exit ' 0 sredata.rxx not found '
  end  /* Do */
  asq=stream('SREDATA.RXX','c','query exists')

  atid=0
  if asq<>' ' then do
      atid=rexxthread('F',asq,usequeue,usesem,adata,delims,nlines, ,
                 fixed,index_format,file_type,btype,cmtchars,varlist,acomment,inoffset)
  end
  if atid=0 then do
       call pmrintf(' SREDATA: Could not start thread for ' main1)
       exit ' 0 could not start thread '
  end  /* Do */
  else do
     call pmprintf('SREDATA: thread launched on 'atid)
  end

  if wordpos(upper(domonitor),'1 YES Y')=0 then exit 1

   apid=dospid()


/* else, monitor what's happening with the daemon */
   say 
   say
   say  " ================================================"
   say "     Monitoring SRE-Data daemon: "aname
   say   'Notes: * daemon launched on thread / process  'atid ' / ' apid
   say   "       * to kill the daemon, hit X  "
   say
   ifoo=0
   do forever
       ifoo=ifoo+1
       if ifoo=18 then do
          say
          say  " ================================================"
          say "     Monitoring SRE-Data daemon: "aname
   say   'Notes: * daemon launched on thread / process  'atid ' / ' apid
          say   "       * to kill the daemon, hit X  "
          say
          ifoo=1
       end
       call syssleep(15)       /* every 15 seconds*/
       foo=rxqueue('s',usequeue)
       foo=queued()
       qmess="  in queue= " foo
       wow=eventsem_query(usesem)
       if wow<0 then do
             if wow=-187 then
                emess=' semaphore does not exist'
             else
                emess=' semaphore error ' wow
       end  /* Do */
       else do
          emess="  pending events=" wow
       end  /* Do */
       say aname " : " qmess " || " emess
       foo=inkey('N')
       if upper(foo)='X' then do
          yow=sref_find_index(targ,'*DIE*'||Aname)
          say " Killing daemon: " yow

          foo=eventsem_reset(usesem)
          foo=eventsem_close(usesem)
          if foo<>0 then say " Problem closing semaphore: " foo
          foo=rxqueue('D',usequeue)
          if foo<>0 then say " Problem closing queue: " foo
           say
           say "Note: to free up resources, close the OS/2 session."
          exit 1
        end
   end /* do */

exit            /* all done with montior */

/*******************/
/* set options, possibly from PROG=.
   ADATA  (possibly not set, since it might be specified
  delims, nlines, fixed, index_format, 
*/
get_opts:              

parse arg aprog,progstring
delims=' ,'     /* 1 char delimiters to parse on */
nlines=1        /* # lines per record */
fixed=0         /* 0 means "not fixed length files */
index_format=0   /* 0 means "parse on a delimiter, not fixed set of characters */
btype=0          /* 0 means "verbatim match */
cmtchars=""    /* "" means "no beginning of line comment characters */
varlist="*"    /* * means "save full observation as the * variable */
crlf='0d0a'x
acomment=""
inoffset=0

if aprog=' ' then return 2

if progstring=1 then do         /* aprog contains the crlf'ed program */
  ii=0
  yow=' program string '
  do until aprog=""
      parse var aprog l1 (crlf) aprog
      ii=ii+1
      progs.ii=l1
  end /* do */
  progs.0=ii
end
else do
   yow=stream(aprog,'c','query exists')
   if yow="" then do
       call pmprintf(' SREDATA: PROG= 'aprog ' not found. ')
       exit '0 PROG not found '
   end  /* Do */
   foo=sref_fileread(yow,progs,,'E')
end

if progs.0=0 then do
     call pmprintf(' SREDATA: PROG= 'yow ' is empty. ')
     exit '0 Prog empty '
end  /* Do */
else do
    call pmprintf(' SREDATA: PROG= ' yow ' has ' progs.0 ' entries ')
end /* do */
do mm=1 to progs.0
   aline=strip(progs.mm)
   if abbrev(aline,';')=1 then iterate
   parse var aline avar '=' aval ; avar=strip(upper(avar)) ;aval=strip(aval)
   select
      when abbrev(avar,'DATA')=1 then adata=aval
      when abbrev(avar,'TYPE')=1 then btype=aval
      when abbrev(avar,'COMMENT')=1 then cmtchars=aval
      when abbrev(avar,'VAR')=1 then varlist=aval
      when abbrev(avar,'DELIM')=1 then delims=decodekeyval(aval)  /* convert %20 to space, etc */

      when abbrev(avar,'LINE')=1 then
         if datatype(aval)='NUM' then nlines=aval

      when abbrev(avar,'FIXED')=1 then
         if datatype(aval)='NUM' then fixed=aval

      when abbrev(avar,'INDEX_F')=1 then index_format=aval
      when abbrev(avar,'HEAD')=1 then acomment=aval
      when abbrev(avar,'IN_OFF')+abbrev(avar,'INOFF')>0 then
         if datatype(aval)='NUM' then inoffset=aval

   otherwise nop
   end  /* select */
end

if btype=' ' then btype=0


return 1


/* procedure from TEST-CGI.CMD by  Frankie Fan <kfan@netcom.com>  7/11/94 */
DecodeKeyVal: procedure
  parse arg Code
  Text=''
  Code=translate(Code, ' ', '+')
  rest='%'
  do while (rest\='')
     Parse var Code T '%' rest
     Text=Text || T
     if (rest\='' ) then
      do
        ch = left( rest,2)
        if verify(ch,'01234567890ABCDEF')=0 then
           c=X2C(ch)
        else
           c=ch
        Text=Text || c
        Code=substr( rest, 3)
      end
  end
  return Text



/********************/
/* check/create queue and semaphore */

make_semq:procedure
parse arg usesem,usequeue
wow=eventsem_query(usesem)
fooq=rxqueue('C',usequeue)
if wow<>-187 & fooq<>usequeue  then return 0

if fooq<>usequeue then foo=rxqueue('d',fooq)  /* clean up */
if wow=-187 then do
  foo=eventsem_create(usesem)
  if foo<>0 then return 0
end /* do */

return 1



/*******************************************/
/* load rexx libraries, etc. */

init_stuff:

foo=rxfuncquery('sysloadfuncs')
if foo=1 then do
  call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'

  call SysLoadFuncs
end
foo=rxfuncquery('rexxlibregister')
if foo=1 then do
 call rxfuncadd 'rexxlibregister','rexxlib', 'rexxlibregister'
 call rexxlibregister
end

if rxfuncquery('sref_fileread')<>0 then do
  ik=rxfuncadd('sref_fileread','rexxlib','LIB_FILEREAD')
  if ik<>0 then do
      call pmprintf(' SREDATA: Warning: unable to register SREF_FILEREAD')
      say' SREDATA: Warning: unable to register SREF_FILEREAD'
  end
end

/* see if SREF_FIND_INDEX in macrospace. IF not, load it (from SREDATA.SRF) */
if macroquery('SREF_FIND_INDEX')='' then do
   foo=macroadd('SREF_FIND_INDEX','SREDATA.SRF','b')
   if foo=0 then do
        call pmprintf(" SREDATA: fatal error, could not load SREF_FIND_INDEX ")
        say " SREDATA: fatal error, could not load SREF_FIND_INDEX "
        exit '0 could not find sref_find_index'
   end
   else do
      call pmprintf(" SREDATA: SREF_FIND_INDEX loaded into macrospace ")
   end  /* Do */
end
else do
     call pmprintf(' SREDATA: SREF_FIND_INDEX already in macrospace ')
end  /* Do */

return 0
