/* SREDATAX: The SRE-http: STM and KEY file creater for the SREDATA addon.
*/
/*************** BEGIN USER CHANGEABLE VARIABLES ********/

/* Verbose: set level of status messages: 0=none,1=a few, 2=a lot */
verbose=2

/* TEMPDIR: set the temporary directory (if blank, use current directory*/
temp_dir="\temp"



/************* END USER CHANGABLE VARIABLES ************/

/* Load up advanced REXX functions */
call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs


/*---- load the rexxlib library */
droprxlib=0 ; gotrxlib=1
foo=rxfuncquery('rexxlibregister')
if foo=1 then do
 droprxlib=1 ; gotrxlib=0
 call rxfuncadd 'rexxlibregister','rexxlib', 'rexxlibregister'
 call rexxlibregister
end
foo=rxfuncquery('rexxlibregister')
if foo=1 then do
    say " Could not find the REXXLIB procedure library (REXXLIB.DLL). "
    say "  Did you download it? "
    exit 0
end  /* Do */

if rxfuncquery('sref_fileread')<>0 then do
  ik=rxfuncadd('sref_fileread','rexxlib','LIB_FILEREAD')
  if ik<>0 then say ' Warning: unable to register SREF_FILEREAD'
end

ansion=checkansi()
if ansion=1 then do
  aesc='1B'x
  cy_ye=aesc||'[37;46;m'
  normal=aesc||'[0;m'
  bold=aesc||'[1;m'
  re_wh=aesc||'[31;47;m'
  reverse=aesc||'[7;m'
end
else do
  say " Warning: Could not detect ANSI....  Install will look ugly ! "
  cy_ye="" ; normal="" ; bold="" ;re_wh="" ;
  reverse=""
end  /* Do */


/*  Check for command line arguments */

aprog="" ; stem_file="" ; key_file=""
doing_cmd=0 ;isstem=0 ; useq=1 ; auseq=1


parse arg anarg


if strip(anarg)="?" then do
   call show_info
   exit 0
end  /* Do */


if anarg<>"" then do            /* parse out variables */
   say " Running SREDATA using command line arguments: "
   say "        "  anarg
   doing_cmd=1
   do until anarg=""
        parse var anarg arg1 anarg
        parse var arg1 avar '=' aval ;
         avar=strip(upper(avar));aval=strip(upper(aval))
        select
           when abbrev(avar,"P")=1 then aprog=aval
           when abbrev(avar,"K")=1 then do
                key_file=aval ; isstem=0
           end
           when abbrev(avar,"Q")=1 then auseq=aval
           when abbrev(avar,"S")=1 then do
              stem_file=aval ; isstem=1
           end
        otherwise nop
        end  /* select */
   end /* do */
   if aprog="" then do
             say " ERROR: no program file specified "
             exit 0
   end
   if stem_file="" & key_file="" then do
             say "ERROR: no stem or key file specified "
             exit 0
   end  /* Do */
   if stem_file<>""  & key_file<>"" then do
                say "ERROR: specify either a stem or key file (not both) "
                exit 0
   end  /* Do */
   if wordpos(aUSEq,'0 N NO')>0 then useq=0
end  /* Do */
else do         /* not command line */

   cls
   say  " " ; say

   call lineout, bold cy_ye
   call lineout, "SREDATAX: An SREDATA utility for creating STEM and KEY files "
   call lineout, normal
   call lineout, cy_ye " Hint. for a complete description, start this program with:"bold" SREDATAX ? " normal
   say " "
end

/* get program file (or was it in the command line argument ? */
if aprog="" then do
  say  bold " Enter the name of the PROGram file " normal
  say " The program file contains information on the structure of your data file"
  call charout," ? "
  parse pull aprog
end

yowp=get_opts(aprog)

oo=lastpos('.',aprog)
if oo="" then 
   basename=aprog
else 
   basename=delstr(aprog,oo)

if adata=' ' then do
    say  'No DATA= specified '
    exit 0
end  /* Do */
yowd=stream(adata,'c','query exists')         /* check on data */
if yowd="" then do
           say 'DATA file= 'adata ' not found. '
           exit 0
end  /* Do */

if doing_cmd=0 then do
  aa=yesno(' Review options read from 'aprog)
   if aa=1 then do
   say bold "Header:"normal acomment
   say bold " DataFile To Read: " normal  adata
   say  bold " Word_delimiters: " re_wh||delims||normal
   say bold "Record Length (0=1 record per line): " normal fixed bold ,
          " : Identifier_Format: " normal index_format
   say bold '#lines per  "non-fixed length" record: ' normal nlines
   SAY BOLD ' Input offset: 'normal inoffset
   say bold "Match_type: " normal atype
   say bold "Comment line characters: " normal comment_chars
   say bold "Variable list: " normal varlist
  end

  isstem=yesno(' Create a Stem or Key file (? for help): ',1,'Stem Key') /*1=stem*/


/* get stem of key file name */
   if isstem=1 then do
      say " "
      say  bold " Enter mame to use for the .STM (stem) file " normal
      uname=basename||'.STM'
      call charout," (Enter="uname ") ? "
      pull stem_file
      if stem_File=""  then stem_file=uname
      tmp=stem_file
      stem_file=stream(stem_file,'c','query exist')
      if stem_file<>' ' then do
         iy=yesno(" Overwrite "STEM_FILE)
         if iy=0 then do
            say " Exiting."
            exit 2
         end
      end
      else do
          stem_file=tmp
      end  /* Do */
   end                          /* istem=1 */
   else do
      say " "
      say  bold " Enter mame to use for the .KEY file " normal
      uname=basename||'.KEY'
      call charout," (Enter="uname ") ? "
      pull key_file
      if key_File=""  then key_file=uname
      tmp=key_file
      key_file=stream(key_file,'c','query exist')
      if key_file<>' ' then do
         iy=yesno(" Overwrite "key_FILE)
         if iy=0 then do
             say " Exiting."
             exit 2
         end
      end
      else do
         key_file=tmp
      end  /* Do */
   end                  /* isstem <>1 */
end                     /* not comand line */

foo=sort_opts(atype)

/* use qsort? */
dsize=stream(adata,'c','query size')
runme=stream('QSORT.EXE','c','query exists')
if doing_cmd=0 & isstem<>1 & runme<>"" then
          USEQ=USE_QSORT(ADATA)  /* use qsort? */

if isstem=0 then do             /* key file: make temporary file*/
  if temp_dir="" then temp_dir=directory()
  oo=strip(temp_dir,'t','\')
  if dosisdir(oo)=0 then do
     say " ERROR: bad temporary directory: " temp_dir
     say " You should set the  TEMP_DIR variable at the top of SREDATAX.CMD"
     exit 0
  end  /* Do */
  tfile=oo||'\sred????.tmp'
  tfile=dostempname(tfile)
  say "Using temporary file: " tfile ; call delay(0.3)
end


/* done getting info -- now create the stem or key file */

if isstem=1 then do   /* make a stem_file */
  foo=read_data(adata,1)  /* read into vals, recnum, and indx */
  foo=sort_me()         /* sort indx (and recnum) */
  foo=save_to_stem(tfile,stem_file,yowd,yowp)
  exit 1
end


if useq=1 then do   /* make a keyfile with qsort */

  ttype='C'
  if pos('I',atype)>0 then ttype='C'
  if Pos('N',atype)>0 then ttype='N'
  if comment_chars<>' ' then ttype=ttype||'D'||comment_chars
  gg='QSORT 'adata' 'tfile' 'inoffset' 0 'fixed' 'fixed_index.1' 'fixed_index.2' 'ttype' 200 0'
  address cmd gg
  oo=stream(tfile,'c','close')

  gpp=charin(tfile,1,200)
  parse var gpp nrecs ksize .
  say " #recs, record_length= " nrecs ksize 
  oo=stream(tfile,'c','close')
  if verbose>1 then say "Writing header information "
  foo=write_key_header(tfile,nrecs,ksize,yowd,yowp,1,fixed,inoffset)
  oo=stream(tfile,'c','close')
  foo=dosrename(tfile,key_file)
  if foo=0 then do
      foo=doscopy(tfile,key_file,'R')
      if foo<>0  then do
          say " ERROR: " foo ". Could not overwrite " key_file
          exit 0
      end
      foo=sysfiledelete(tfile)
      if foo<>0 then say "Warning: could not delete ("foo") temporary file "tfile
  end
  foo=sort_2(key_file,200,nrecs,ksize,ttype)


end
else do   /* make a keyfile using arraysort */
  foo=read_data(adata,0)  /* read into vals, recnum, and indx */
  foo=sort_me()         /* sort indx (and recnum) */
  foo=save_to_key(tfile,key_file,yowd,yowp)
end
    

exit 1

/***********/
/* sort on record number */
sort_2:procedure
parse arg afile,noff,jrecs,jsize,datype
jsize=jsize+4
foo=stream(afile,'c','open')
if abbrev(upper(foo),'READY')<>1 then do
        say  " Could not open file, no 2nd level sort by record number "
        exit 0
end  /* Do */

inlist=1
vv=charin(afile,noff+1,jsize)
ivv=c2d(left(vv,4)); oldv=substr(vv,5);

say " 2nd sort on record number ... "
do mm=2 to jrecs
    if verbose>1 &  mm//5000 =1 then say ' ... at # 'mm

    i0=(noff+1)+( (mm-1)*jsize) 
    vv=charin(afile,i0,jsize)
    ivv=c2d(left(vv,4)); avv=substr(vv,5);notsame=0
    if datype='C' then do               /* exact match */
         if oldv\==avv then notsame=1
    end
    else do                             /* numeric ok match */
         if oldv\=avv then notsame=1
    end  /* Do */
    if notsame=1 & inlist=1 then do
         gotems.inlist=left(ivv,10)||vv
         oldv=avv
         i0old=i0
         iterate /* one with same, leave as is */
    end
 
    if notsame=0 then do        /* append to list */
        inlist=inlist+1
        gotems.inlist=left(ivv,10)||vv
    end  /* Do */
    else do                     /* got list of sames, sort and write */
        gotems.0=inlist
        foo=arraysort(gotems,,,1,10,,'N')
        do ii=1 to inlist
            a1=left(gotems.ii,10); 
            a2=substr(gotems.ii,11)
            i02=i0old+((ii-1)*jsize)
            foo=charout(afile,a2,i02)
        end /* do */
        oldv=avv ; i0old=i0 ; inlist=1
        gotems.inlist=left(ivv,10)||vv

    end  /* Do */
end

return 1




/*******************/
/* set options, possibly from PROG=.
   ADATA  (possibly not set, since it might be specified
  delims, nlines, fixed, index_format, filetype
*/
get_opts:              

parse arg aprog

acomment=""     /* header comment */
delims=' ,'     /* 1 char delimiters to parse on */
nlines=1        /* # lines per record */
fixed=0         /* 0 means "not fixed length files */
index_format=0   /* 0 means "parse on a delimiter, not fixed set of characters */
filetype='R'    /* regular file type */
atype=0          /* 0 means "verbatim match */
comment_chars=""    /* "" means "no beginning of line comment characters */
varlist="*"    /* * means "save full observation as the * variable */
inoffset=0

if aprog=' ' then return 'defaults'

yow=stream(aprog,'c','query exists')
if yow="" then do
     say ' PROGram file= 'aprog ' not found. '
     exit 0
end  /* Do */

foo=sref_fileread(yow,progs,,'E')   

if foo=0 then do
     say ': PROG= 'yow ' is empty. '
     exit 0
end  /* Do */
else do
    say ': PROG= ' yow ' has ' progs.0 ' entries '
end /* do */
do mm=1 to progs.0
   aline=strip(progs.mm)
   if abbrev(aline,';')=1 then iterate
   parse var aline avar '=' aval ; avar=strip(upper(avar)) ;aval=strip(aval)
   select
      when abbrev(avar,'DATA')=1 then adata=aval
      when abbrev(avar,'TYPE')=1 then atype=aval
      when abbrev(avar,'COMMENT')=1 then comment_chars=aval
      when abbrev(avar,'HEAD')=1 then acomment=aval

      when abbrev(avar,'FILE_T')=1 then filetype=aval
      when abbrev(avar,'VAR')=1 then varlist=aval
      when abbrev(avar,'DELIM')=1 then delims=decodekeyval(aval)  /* convert %20 to space, etc */
      when abbrev(avar,'LINE')=1 then
         if datatype(aval)='NUM' then nlines=aval
      when abbrev(avar,'IN_OFF')+abbrev(avar,'INOFF')>0 then
         if datatype(aval)='NUM' then inoffset=aval


      when abbrev(avar,'FIXED')=1 then
         if datatype(aval)='NUM' then fixed=aval
      when abbrev(avar,'INDEX_F')=1 then index_format=aval
   otherwise nop
   end  /* select */
end

if atype=' ' then atype=0



/* format for reading identifier (index variable) */
fixed_index=index_format
if fixed_index=0 | fixed_index="" then do
    fixed_index.1=0 ; fixed_index.2=0
end
else do
   parse var fixed_index v1 v2 ; v1=strip(v1); v2=strip(v2)
   if datatype(v1)<>'NUM' | (v2<>"" & datatype(v2)<>'NUM') then do
       say 'Bad values in INDEX_FORMAT statement:'fixed_index
       exit 0
   end
   fixed_index.1=v1 ; fixed_index.2=v2
   if fixed_index.2=' ' then fixed_index.2=0
end



return yow



/* procedure from TEST-CGI.CMD by  Frankie Fan <kfan@netcom.com>  7/11/94 */
DecodeKeyVal: procedure
  parse arg Code
  Text=''
  Code=translate(Code, ' ', '+')
  rest='%'
  do while (rest\='')
     Parse var Code T '%' rest
     Text=Text || T
     if (rest\='' ) then
      do
        ch = left( rest,2)
        if verify(ch,'01234567890ABCDEF')=0 then
           c=X2C(ch)
        else
           c=ch
        Text=Text || c
        Code=substr( rest, 3)
      end
  end
  return Text



/************************/
/* create indx. recnum, and vars. */

read_data:

parse arg adata,isstem

vars.0=0
oo=""

/* load file into a stem variable */

if fixed=0 then do              /* crlf delimited records */
    foo=sref_fileread(adata,val1a,,'E')
    iva=0
    JB=1+INOFFSET
    do mm=JB to val1a.0
            if val1a.mm="" then iterate
            aa=left(strip(val1a.mm),1)
            if pos(aa,comment_chars)>0 then iterate /* drop this line */
            iva=iva+1
            val1.iva=val1a.mm
    end
    val1.0=iva ; drop val1a.
    if val1.0=0 then do
       say ' Failed to read data from 'adata
       exit 0
    end
    if verbose>1 then say ' # lines read from 'adata' = 'foo
    if nlines>1 then do
        iarf=0
        oo='(after multiline correction)'
        do mm=1 to (val1.0/nlines)
            aa="" ;       
            ist=((mm-1)*nlines)+1 ; ie=ist+nlines-1
            do mm2=ist to ie
                aa=aa||val1.mm2
            end
            iarf=iarf+1
            val1.iarf=aa
        end
        val1.0=iarf       /* note: if not enuf lines, odd crap at end of last record */
     end
end
else do                 /* fixed length records */
  IOFF=INOFFSET*FIXED
  aa1=stream(adata,'c','close')
  aa2=stream(adata,'c','open')
  if abbrev(upper(aa2),'READY')=0 & aa<>' ' then do
       say ' Could not open file 'adata ',' aa1', 'aa2
       exit 0
  end

  foo=charin(adata,1+IOFF,dosfsize(adata)-(ioff))
  aa=stream(adata,'c','close')

  if foo="" then do
       say ' Failed, to read data from 'adata
       exit 0
  end
  mm0=0
  if Verbose>0 then say "Fixed length read, # chars=: " length(foo)

  do mm=1 to length(foo) by fixed
        mm0=mm0+1
        val1.mm0=substr(foo,mm,fixed)
  end
  val1.0=mm0
end


/* We now have all records in VAL1: now break into  vars, and indx */

/* first, extract index variable */

if fixed_index.2>0 then do     /* fixed location index field */
   if verbose>1 then say "Extracting fixed index  from # recs "oo": "val1.0
    do mm=1 to val1.0
      indx.mm=substr(val1.mm,fixed_index.1,fixed_index.2)
    end
    index_max=fixed_index.2
end
else do                        /* delimited index field */
   if verbose>1 then say "Extracting  delimited index  from # recs "oo": "val1.0
    index_max=0
    do mm=1 to val1.0
        if verbose>1 &  mm//1000 =1 then say ' ... at # 'mm
        if delims="" & fixed_index.1<=1 then do   /* quick entry for special case*/
             parse var val1.mm indx.mm .
        end
        else do
           cline=multi_parse(val1.mm,delims)
          lp=max(1,fixed_index.1)
           if lp<=parsed.0 then
              indx.mm=strip(parsed.lp)
           else
              indx.mm=""
        end
        index_max=max(index_max,length(indx.mm))
    end
end
indx.0=val1.0


/* 2nd: extract "variables". Note: * variable means "entire record" */
/* but don't bother if doing a key file */

if varlist="" | varlist=0 | isstem=0 then do   /* none specified, so return */
  vars.0=0
  all_var=0
  return  1
end

/* create variable list */
nvars=0 ; all_var=0
do until varlist=""
    parse var varlist v1 ','  varlist
    if strip(v1)='*' then do            /* * means "entire record */
        nvars=nvars+1
        vars.nvars='*' ; all_var=nvars
        iterate
    end
    else do                             /* portion of record */
        parse var v1 mname  iat  ilen ; iat=strip(iat) ;ilen=strip(ilen)
        if datatype(iat)<>'NUM' then iterate
        if ilen="" then ilen=0
        if  datatype(ilen)<>'NUM' then iterate
        nvars=nvars+1
        vars.nvars=strip(translate(mname)) 
        vars.nvars.!iat=iat ; vars.nvars.!ilen=ilen
    end
end                     /* end of varlist */
vars.0=nvars
if verbose>1 then say 'Extracting 'nvars' variables'

/* extract variables from each observation */

do ww=1 to indx.0
  mud=0
  if verbose>1 &  ww//1000 =0 then say  ' ... at # 'ww
  do jv=1 to nvars
    select
      when  vars.jv='*' then do                /* entire record */
           vars.ww.jv=val1.ww
      end
      when vars.jv.!ilen>0 then do     /* interpret as start, length */
          vars.ww.jv=substr(val1.ww,vars.jv.!iat,vars.jv.!ilen)
      end
      otherwise  do        /* interpet as nth word */
       if mud=0 then cline=multi_parse(val1.ww,delims) /* avoid multiple parses, same record */
       mud=1
       lt=vars.jv.!iat
       if lt<=parsed.0 then
                vars.ww.jv=parsed.lt
           else
                vars.ww.jv=''

      end         /* OTHERWISE (do  delimited type */
    end         /* select */
  end           /* do jv */
end             /* do ww */

return 1


/*************/
/* parse a line, using multiple delimiters, return in stem var parsed. */

multi_parse:procedure expose parsed.

parse arg aline,dlm
it=0

if pos(' ',dlm)>0 | dlm="" then do
   aline=translate(aline,' ',dlm)
   do until aline=""
      it=it+1
      parse var aline parsed.it aline
   end
   parsed.0=it
end
else do
   dl1=left(dlm,1); dl2=copies(dl1,length(dlm))
   aline=translate(aline,dl2,dlm)
   do until aline=""
        parse var aline v1 (dl1) aline
        if length(v1)<>0 then do
           it=it+1 ; parsed.it=v1
        end
   end
end
return it


/********/
/* set some sort thingies */
sort_opts:procedure expose dowild dolc dost numeric_match
parse arg atype
numeric_match=0         /* assume use of == instead of = */
if pos('N',atype)>0 then numeric_match=1  /* use num_isearch instead of isearch */

dolc=0 ; dost=0 ; dowild=0
if pos('*',atype)>0 then dowild=1
if pos('I',atype)>0 | dowild=1 then dolc=1
if pos('S',atype)>0 | dowild=1 then dost=1


return 1



/***************************/
/* save to key file */

save_to_key:procedure expose atype indx. recnum. fixed_index. index_max ,
        normal reverse bold acomment inoffset fixed

parse arg tmpfile,kfile,datafile,progfile

if verbose>1  then say "Preparing to save to key file (# records= "indx.0

foo=write_key_header(tmpfile,indx.0,index_max,datafile,progfile,,fixed,inoffset)

/* now write each index value */
say " Writing key file records to " tmpfile

do mm=1 to indx.0
   if mm//1000=0 then say " ... writing record # " mm
   t1=right(x2c(d2x(recnum.mm)),4,'00'x)   /* 10billion record max? */
    if isfixed>0 then do                /* fixed length index values */
        t3=t1||indx.mm
    end
    else do             /* variable length index values */
        l4=length(indx.mm) 
        t2=right(x2c(d2x(length(indx.mm))),2,'00'x)   /* 4k max index size */
        t3=t1||t2||left(indx.mm,index_max)
    end
    foo=charout(tmpfile,t3)
    if foo>0 then do
         say foo " ERROR writing header-record to " tmpfile
         exit 0
    end
end
foo=stream(tmpfile,'c','close')

foo=dosrename(tmpfile,kfile)    /* rename ?*/
if foo=0 then do                /* no -- possibly differnet drive , or already exists */
    foo=doscopy(tmpfile,kfile,'R')  /* repalce mode */
    if foo<>0 then do
       say " ERROR: could not copy " tmpfile " to " kfile
       exit 0
    end  /* Do */
    foo=sysfiledelete(tmpfile)
end  /* Do */


return 1



/****************************************/
write_key_header:procedure expose isfixed atype  acomment fixed_Index. ,
                                 bold normal reverse indx.
parse arg kfile2,nrecs,index_max,datafile,progfile,asis,recl,noff

hello='SREFKEYFILE'
creation=time('n')' 'date('n')
if length(datafile)>30 then datafile=right(datafile,35)
if length(progfile)>30 then progfile=right(progfile,35)

/* isfixed =0 only if "word" read (fixed_index.2=0, and NOT qsorted */

if asis=1 then do
     isfixed=index_max
     record_length=index_max+4
end  /* Do */
else do
  isfixed=0
  record_length=index_max+6
  if fixed_index.2>0 then do
     isfixed=fixed_index.2
     record_length=index_Max +4
  end /* do */
end
aa=left(hello','nrecs','record_length','isfixed','atype','creation','datafile' 'recl' 'noff','progfile','acomment,200)
foo=charout(kfile2,aa,1)
if foo<>0 then do
    say foo" ERROR writing header-record to " kfile
    exit 0
end  /* Do */

return 1

    
/***************************/
/* read data from stem file 

   Stem_file is organized as:
      v.n = identifier-value (the indx)
      v.n.!rec =  record # in original file (of v.n)
      v.n.ivar = ivar=1..nvars optional "variables"
      !vars.ivar=  ivar=1..nvars variable names
      !type = Sort/search type 
 
*/

save_to_stem:procedure expose vars. indx. verbose recnum. all_var atype ,
                        normal reverse bold re_wh acomment
parse arg tmpfile,stem_file,datafile,progfile

drop val1.
if verbose>1  then say "Preparing to save to stem file (# records= "indx.0', # variables='vars.0
do mm=1 to indx.0
   val1.mm=indx.mm
   val1.mm.!rec=recnum.mm
   do ibo=1 to vars.0
       val1.mm.ibo=vars.mm.ibo
   end
end
val1.0=indx.0
val1.!vars.0=vars.0
do iy=1 to vars.0               /* get the varnames */
  val1.!vars.iy= vars.iy
end

say " Saving sort type: " atype
/* other info */
val1.!type=atype
val1.!creation=time('n')' 'date('n')
val1.!datafile=datafile
val1.!progfile=progfile
val1.!comment=acomment

say " Saving to Stem file: " stem_file
if stream(stem_file,'c','query exists')<>' ' then do
    foo=sysfiledelete(stem_File)
    if foo<>0 then do
        say " ERROR: " foo ". Could not delete old stem file: " stem_file
    end  /* Do */
end  /* Do */

foo=cvwrite(stem_file,val1)
 
 if foo=0 then 
     say "Warning, could not create " stem_files
 else
      if verbose>0 then say '  STEM file written: 'stem_file

drop val1.

return 1

/**********/
/* given indx, sort indx (and create recnum.  */

sort_me:procedure expose indx. recnum. verbose dolc numeric_match

if verbose>1 then say "About to sort index, # elements= "indx.0
do mm=1 to indx.0
   tmps.mm=left(mm,8)||' '||indx.mm
end
tmps.0=indx.0
/* now sort */
arg7="C"
if dolc=1 then arg7='I'
if numeric_match=1 then arg7='N'
signal off syntax ; signal off error
signal on error name badsort ; signal on syntax name badsort
stat=arraysort(tmps,,,10,,,arg7,1,8,,'N')  /* note seconday sort on recnum */
signal on error name oo ; signal on syntax name oo

if verbose>1 then say 'Sort is done, status= 'stat
if stat=0 then do
   say ' Error sorting identifiers '
   exit 0
end

do mm=1 to tmps.0
   indx.mm=substr(tmps.mm,10)
   oo=strip(substr(tmps.mm,1,8))
   recnum.mm=oo
end
if verbose>1 then say 'Setting up sorted index: done. '

drop tmps. 
return 0

badsort:
call pmprintf(' Error in sort ('stat'), possibly non numeric element? ')
exit 0

/****************/
/* USE QSORT QUERYER? */
USE_QSORT:
PARSE ARG ADATA

dsize=dosfsize(adata)
runme=stream('QSORT.EXE','c','query exists')
if runme="" then return 0
say 
SAY " For larger  datasets, the REXXLIB ARRAYSORT procedure used here can be "
say " very slow. Instead, you can use QSORT (an external EXEcutable)."
say  " Although QSORT is much faster, it does not understand multiple lines, "
say  " or non-standard delimiters (i.e.; other then space and comma)."
say  " NOTE: the size of your dataset= "dsize
aa=yesno(' Use QSORT? ')
return aa




 /* ------------------------------------------------------------------ */
 /* function: Check if ANSI is activated                               */
 /*                                                                    */
 /* call:     CheckAnsi                                                */
 /*                                                                    */
 /* where:    -                                                        */
 /*                                                                    */
 /* returns:  1 - ANSI support detected                                */
 /*           0 - no ANSI support available                            */
 /*          -1 - error detecting ansi                                 */
 /*                                                                    */
 /* note:     Tested with the German and the US version of OS/2 3.0    */
 /*                                                                    */
 /*                                                                    */
 CheckAnsi: PROCEDURE
   thisRC = -1

   trace off
                         /* install a local error handler              */
   SIGNAL ON ERROR Name InitAnsiEnd

   "@ANSI 2>NUL | rxqueue 2>NUL"

   thisRC = 0

   do while queued() <> 0
     queueLine = lineIN( "QUEUE:" )
     if pos( " on.", queueLine ) <> 0 | ,                       /* USA */
        pos( " (ON).", queueLine ) <> 0 then                    /* GER */
       thisRC = 1
   end /* do while queued() <> 0 */

 InitAnsiEnd:
 signal off error
 RETURN thisRC


/* -------------------- */
/* get a yes or no , return 1 if yes */
yesno:procedure expose normal reverse bold
parse arg fooa , allopt,altans
if altans<>" " & words(altans)>1 then do
   w1=strip(word(altans,1))
   w2=strip(word(altans,2))
   a1=left(w1,1) ; a2=left(w2,1)
   a1a=substr(w1,2) ; a2a=substr(w2,2)
end
else do
    a1='Y' ; a1a='es'
    a2='N' ; a2a='o'
end  /* Do */
ayn='  '||bold||a1||normal||a1a||'\'||bold||a2||normal||a2a
if allopt=1 then  ayn=ayn||'\'||bold||'?'||normal

do forever
 foo1=normal||reverse||fooa||normal||ayn
 call charout,  foo1 normal ':'
 pull anans
 if abbrev(anans,a1)=1 then return 1
 if abbrev(anans,a2)=1 then return 0
 if allopt=1 & abbrev(anans,'?')=1 then do
say
say " The SREDATA addon for SRE-http can read three types of files: "
say bold "1) DATA files " normal
say "   DATA files are the files containing your raw data.  These "
say "   are typically, but not necessarily, text files."
say "   The instructions in your SREDATA PROGram file are used to create"
say "   these DATA files. "
say bold "2) STEM files" normal
say "   STEM (.STM) files are special binary transforms of DATA files. More "
say "   precisely, they have been sorted (on the identifier variable) "
say "   and formatted for quick input. "
say bold "3) KEY files  " normal
say "   KEY (.KEY) files are special (binary) extracts from DATA files. "
say "   They contain ONLY the sorted values of the identifier variables. "
say "   They are meant to be used when the size of a database precludes use "
say "   of a STEM or a DATA file."
say
 end  /* Do */
end

/**********/
/* show description */
show_info:

cls;
   call lineout, bold cy_ye
   call lineout, "SREDATAX: An SREDATA utility for creating STEM and KEY files "
   call lineout, normal
  
  say bold "SREDATAX " normal " is a stand-alone portion of the SRE-DATA addon for"
  say " the SRE-http web server.  It is used to produce "bold "STEM" normal "and " bold"KEY"normal" files."
say 
  say " To run in batch mode, the calling syntax is: "
  Say bold "     SREDATAX  P=progfile.nam t=filename.ext Q=yn" normal
  say  " where t=K(eyfile) or S(temfile) "

call charout , cy_ye ' (hit any key to continue, X to exit) ' normal
 foo=sysgetkey("noecho")
 if strip(upper(foo))="X"  then exit
 say
 say


  say cy_ye"                 Introduction                       "normal
  say " You can run SREDATAX in keyboard-input or batch mode.  To run in keyboard "
  say " mode, just enter SREDATAX from an OS/2 prompt, and then answer the questions."
  say
  say " To run in batch mode, use the following syntax:"
  Say  "     SREDATAX  P=progfile.nam t=filename.ext Q=yn"
  say " Where: "
  say "     P = the "bold"PROG"normal"ram file.  PROGram files contain "
  say "         the what (and how) of the data to be read"
  say "     T = K or S = Keyfile or Stemfile-- use one or the other (not both)."
  say "         A Key, or Stem, file will be written to filename.ext "
  say "     Q = If yn=1, then use the QSORT program when creating KEYFILES. "
  say "         On large files, QSORT is much faster; however in some cases"
  say "         (such as when odd delimiters are used), it should not be used."


 call charout , cy_ye ' (hit any key to continue) ' normal
 foo=sysgetkey("noecho")
 say
 say

  say cy_ye"                 Types of files                       "normal
 say " SREDATAX creates two special files that SRE-DATA can read.  "
 say " Either of these are generated from the combination of PROGram "
 say " file and a DATA file".
say bold "i) DATA files " normal
say "   DATA files are the files containing your raw data.  These "
say "   are typically, but not necessarily, text files."
say bold"ii)PROGram files "normal
say "   PROGram file contain format instructions (including the name of "
say "   a DATA file).  These instructions include record size, identifier, "
say "   and delimiter information."

say
call charout , cy_ye ' (hit any key to continue) ' normal
 foo=sysgetkey("noecho")
 say

say " Although SRE-Data can use these files as is, it is often convenient "
say " to transform the DATA file into a more easily used format. In particular,"
say " SRE-Data can read STEM or KEY files:"
say bold "1) STEM files" normal
say "   STEM (.STM) files are special binary transforms of DATA files. More "
say "   precisely, they have been sorted (on the identifier variable), "
say "   and formatted for quick input. "
say bold "2) KEY files  " normal
say "   KEY (.KEY) files are special (binary) extracts from DATA files. "
say "   They contain ONLY the sorted values of the identifier variables. "
say "   They are meant to be used when the size of a database precludes use "
say "   of a STEM or a DATA file."

 call charout , cy_ye ' (hit any key to continue) ' normal
 foo=sysgetkey("noecho")
 say
 
 say cy_ye"                 Notes                    "normal
 say " *  SREDATAX contains two parameters that users may change: "
 say "    i) VERBOSE -- controls how many status messages are displayed"
 say "   ii) TEMP_DIR -- temporary file directory "
 say "  *  If you specify an existing Stem or Key file, it will be overwritten."
 say " *  For more information on SRE-Data, see the SREDATA.HTM file that "
 say "    comes with SRE-DATA. "


 call charout , cy_ye ' (hit any key to exit) ' normal
 foo=sysgetkey("noecho")
 say

return 0


  say
